package com.cxytiandi.yunpan.service;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.cxytiandi.yunpan.document.Resource;
import com.cxytiandi.yunpan.repository.ResourceRepository;
import com.cxytiandi.yunpan.util.FileUtils;
import com.cxytiandi.yunpan.util.ZipUtil;
import com.cxytiandi.yunpan.vo.ResourceVo;
import com.mongodb.gridfs.GridFSDBFile;

@Service
public class ResourceServiceImpl implements ResourceService {
	
	@Autowired
	private ResourceRepository resourceRepository;
	
	@Autowired
	private FileService fileService;
	
	@Value("${upload.file.path}")
	private String uploadFilePath;
	
	@Override
	public List<ResourceVo> list(String parentId) {
		List<ResourceVo> results = new ArrayList<ResourceVo>();
		List<Resource> list = resourceRepository.findAllByParentId(parentId);
		for (Resource resource : list) {
			ResourceVo vo = new ResourceVo();
			BeanUtils.copyProperties(resource, vo);
			vo.setForamtSize(FileUtils.getDataSize(resource.getSize()));
			if (resource.getType() == 0) {
				vo.setIcon("images/folder.png");
			} else {
				vo.setIcon(FileUtils.getIcon(resource.getName()));
			}
		
			results.add(vo);
		}
		
		return results;
	}

	@Override
	public void rename(String id, String name) {
		Resource resource = resourceRepository.findOne(id);
		if (resource != null) {
			resource.setName(name);
			resourceRepository.save(resource);
		}
	}

	@Override
	public void remove(String[] ids) {
		List<Resource> res = this.listByParentIds(Arrays.asList(ids));
		for (Resource re : res) {
			resourceRepository.delete(re);
			if (re.getType() == 1) {
				try {
					fileService.removeFile(re.getFileId());
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}

	@Override
	public void save(Resource r) {
		resourceRepository.save(r);
	}

	@Override
	public Resource get(String id) {
		return resourceRepository.findOne(id);
	}

	@Override
	public List<Resource> listByParentIds(List<String> parIds) {
		List<Resource> results = new ArrayList<Resource>();
		for (String pid : parIds) {
			results.add(resourceRepository.findOne(pid));
			doListForBranchIds(results, pid);
		}
		
		return results;

	}
	
	private void doListForBranchIds(List<Resource> results, String id) {
		List<Resource> resources = resourceRepository.findByParentId(id);
		for (Resource r : resources) {
			if(!results.contains(r)) {
				results.add(r);
				doListForBranchIds(results, r.getId());
			}
		}
	}

	@Override
	public String downLoadFileToZip(List<String> parIds) {
		try {
			String uuid = UUID.randomUUID().toString().replaceAll("-", "");
			String rootPath = uploadFilePath + uuid;
			Files.createDirectory(Paths.get(rootPath));
			for (String pid : parIds) {
				Resource r = resourceRepository.findOne(pid);
				if (r.getType() == 0) {
					Files.createDirectory(Paths.get(rootPath + File.separator + r.getName()));
					doListForBranchIds2(pid, rootPath + File.separator + r.getName());
				} else {
					GridFSDBFile fs = fileService.getFile(r.getFileId());
					Files.copy(fs.getInputStream(), Paths.get(rootPath + File.separator + r.getName()));
					doListForBranchIds2(pid, rootPath);
				}
			}
			File zipFile = ZipUtil.zip(rootPath);
			return zipFile.getAbsolutePath();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	private void doListForBranchIds2(String id, String parentPath) throws Exception {
		List<Resource> resources = resourceRepository.findByParentId(id);
		for (Resource r : resources) {
			if (r.getType() == 0) {
				Files.createDirectory(Paths.get(parentPath + File.separator + r.getName()));
				doListForBranchIds2(r.getId(), parentPath + File.separator + r.getName());
			} else {
				GridFSDBFile fs = fileService.getFile(r.getFileId());
				Files.copy(fs.getInputStream(), Paths.get(parentPath + File.separator + r.getName()));
				doListForBranchIds2(r.getId(), parentPath);
			}
		}
	}

}
