package com.cxytiandi.yunpan.web;

import java.io.InputStream;
import java.io.OutputStream;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Date;
import java.util.Map;
import java.util.UUID;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.multipart.MultipartFile;

import com.cxytiandi.yunpan.document.File;
import com.cxytiandi.yunpan.document.Resource;
import com.cxytiandi.yunpan.service.FileService;
import com.cxytiandi.yunpan.service.ResourceService;
import com.cxytiandi.yunpan.util.JsonResult;
import com.mongodb.gridfs.GridFSDBFile;

/**
 * 文件控制器
 * @author yinjihuan
 */
@Controller
public class FileController {
	
	@Autowired
	private ResourceService resourceService;
	
	@Autowired
	private FileService fileService;
	
	/**
	 * 文件列表初始化页面
	 * @author yinjihuan
	 * @param model
	 * @return
	 */
	@GetMapping("/files")
	public String filesPage(Map<String, Object> model) {
		model.put("files", resourceService.list(null));
		model.put("parentId", "");
		return "files";
	}
	
	/**
	 * 某个文件夹下面的文件列表
	 * @author yinjihuan
	 * @param parentId
	 * @param model
	 * @return
	 */
	@GetMapping("/files/{parentId}")
	public String filesPage(@PathVariable String parentId, Map<String, Object> model) {
		model.put("files", resourceService.list(parentId));
		model.put("parentId", parentId);
		return "files";
	}
	
	/**
	 * 重命名
	 * @author yinjihuan
	 * @param id
	 * @param name
	 * @return
	 */
	@PostMapping("/files/rename")
	@ResponseBody
	public JsonResult rename(String id, String name) {
		resourceService.rename(id, name);
		return new JsonResult();
	}
	
	/**
	 * 删除
	 * @author yinjihuan
	 * @param id
	 * @return
	 */
	@PostMapping("/files/remove")
	@ResponseBody
	public JsonResult remove(String id) {
		resourceService.remove(new String[]{id});
		return new JsonResult();
	}
	
	/**
	 * 批量删除
	 * @author yinjihuan
	 * @param ids
	 * @return
	 */
	@PostMapping("/files/removes")
	@ResponseBody
	public JsonResult remove(@RequestParam(value="ids[]", required=true) String[] ids) {
		if (ids != null) {
			resourceService.remove(ids);
		}
		return new JsonResult();
	}
	
	/**
	 * 创建文件夹
	 * @author yinjihuan
	 * @param name
	 * @param parentId
	 * @return
	 */
	@PostMapping("/folder/create")
	@ResponseBody
	public JsonResult createFolder(String name, String parentId) {
		Resource r = new Resource();
		r.setCreateDate(new Date());
		r.setName(name);
		if (!StringUtils.hasText(parentId)) {
			r.setParentId(null);
		} else {
			r.setParentId(parentId);
		}
		r.setSize(0L);
		r.setType(0);
		resourceService.save(r);
		return new JsonResult();
	}
	
	/**
	 * 文件上传
	 * @author yinjihuan
	 * @param file
	 * @param request
	 * @return
	 */
	@RequestMapping(value = "/files/upload", method = RequestMethod.POST)
	@ResponseBody
	public String uploadFile(@RequestParam("file") MultipartFile file, @RequestParam("parentId") String parentId, HttpServletRequest request) {
		try {
			InputStream content = file.getInputStream();
			File fileVo = new File(file.getOriginalFilename(), content);
			String fileId = fileService.saveFile(fileVo);
			Resource resource = new Resource();
			resource.setCreateDate(new Date());
			resource.setFileId(fileId);
			resource.setName(file.getOriginalFilename());
			resource.setParentId(parentId.equals("") ? null : parentId);
			resource.setType(1);
			resource.setSize(fileService.getFile(fileId).getLength());
			resourceService.save(resource);
			String result = "{fileId:\""+fileId+"\"}";
			return result;
		} catch (Exception e) {
		}
		return null;
	}
	
	/**
	 * 下载文件
	 * @param fileId
	 * @param response
	 */
	@RequestMapping(value = "/files/download/{id}")
	@ResponseBody 
	public void getImage(@PathVariable String id, HttpServletResponse response) {     
	     try {
	    	 Resource resource = resourceService.get(id);
	    	 if (resource != null) {
	    		 if (resource.getType() == 1) {
	    			 GridFSDBFile gridfs = fileService.getFile(resource.getFileId());
			         if (gridfs != null) {
			        	 response.setContentType("application/octet-stream");
				         OutputStream out = response.getOutputStream();
				         response.setHeader("Content-disposition","attachment; filename="+gridfs.getFilename());
				         gridfs.writeTo(out);        
				         out.flush();        
				         out.close();
					 }
				 } else {
					 String zipFile = resourceService.downLoadFileToZip(Arrays.asList(resource.getId()));
					 response.setHeader("Content-disposition","attachment; filename="+UUID.randomUUID().toString().replaceAll("-", "")+".zip");
					 OutputStream out = response.getOutputStream();
			         Files.copy(Paths.get(zipFile), out);
			         out.flush();        
			         out.close();
				 }
			 }
	     } catch (Exception e) {
	          e.printStackTrace();
	     }
	 }

	@RequestMapping(value = "/files/downloads")
	@ResponseBody
    public void downloadFiles(String ids, HttpServletRequest request, HttpServletResponse response){
		try {
			String[] array = ids.split("_");
			String zipFile = resourceService.downLoadFileToZip(Arrays.asList(array));
			response.setHeader("Content-disposition","attachment; filename="+UUID.randomUUID().toString().replaceAll("-", "")+".zip");
			OutputStream out = response.getOutputStream();
	        Files.copy(Paths.get(zipFile), out);
	        out.flush();        
	        out.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
	
}
